@@ -1,10 +1,11 @@
 package com.genymobile.scrcpy.wrappers;
 
-import android.content.res.Configuration;
-import android.graphics.Rect;
-import android.view.IDisplayWindowListener;
+import com.genymobile.scrcpy.util.Ln;
 
-import java.util.List;
+import android.content.res.Configuration;
+import android.os.Parcel;
+import android.os.RemoteException;
+import android.view.IDisplayWindowListener;
 
 public class DisplayWindowListener extends IDisplayWindowListener.Stub {
     @Override
@@ -23,32 +24,14 @@ public class DisplayWindowListener extends IDisplayWindowListener.Stub {
     }
 
     @Override
-    public void onFixedRotationStarted(int displayId, int newRotation) {
-        // empty default implementation
-    }
-
-    @Override
-    public void onFixedRotationFinished(int displayId) {
-        // empty default implementation
-    }
-
-    @Override
-    public void onKeepClearAreasChanged(int displayId, List<Rect> restricted, List<Rect> unrestricted) {
-        // empty default implementation
-    }
-
-    @Override
-    public void onDesktopModeEligibleChanged(int displayId) {
-        // empty default implementation
-    }
-
-    @Override
-    public void onDisplayAddSystemDecorations(int displayId) {
-        // empty default implementation
-    }
-
-    @Override
-    public void onDisplayRemoveSystemDecorations(int displayId) {
-        // empty default implementation
+    public boolean onTransact(int code, Parcel data, Parcel reply, int flags) throws RemoteException {
+        try {
+            return super.onTransact(code, data, reply, flags);
+        } catch (AbstractMethodError e) {
+            Ln.v("Ignoring AbstractMethodError: " + e.getMessage());
+            // Ignore unknown methods, write default response to reply parcel
+            reply.writeNoException();
+            return true;
+        }
     }
 }
